/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	"github.com/rancher/lasso/pkg/client"
	"github.com/rancher/lasso/pkg/controller"
	"github.com/rancher/wrangler/pkg/generic"
	v1 "k8s.io/api/rbac/v1"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type RoleHandler func(string, *v1.Role) (*v1.Role, error)

type RoleController interface {
	generic.ControllerMeta
	RoleClient

	OnChange(ctx context.Context, name string, sync RoleHandler)
	OnRemove(ctx context.Context, name string, sync RoleHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() RoleCache
}

type RoleClient interface {
	Create(*v1.Role) (*v1.Role, error)
	Update(*v1.Role) (*v1.Role, error)

	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1.Role, error)
	List(namespace string, opts metav1.ListOptions) (*v1.RoleList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.Role, err error)
}

type RoleCache interface {
	Get(namespace, name string) (*v1.Role, error)
	List(namespace string, selector labels.Selector) ([]*v1.Role, error)

	AddIndexer(indexName string, indexer RoleIndexer)
	GetByIndex(indexName, key string) ([]*v1.Role, error)
}

type RoleIndexer func(obj *v1.Role) ([]string, error)

type roleController struct {
	controller    controller.SharedController
	client        *client.Client
	gvk           schema.GroupVersionKind
	groupResource schema.GroupResource
}

func NewRoleController(gvk schema.GroupVersionKind, resource string, namespaced bool, controller controller.SharedControllerFactory) RoleController {
	c := controller.ForResourceKind(gvk.GroupVersion().WithResource(resource), gvk.Kind, namespaced)
	return &roleController{
		controller: c,
		client:     c.Client(),
		gvk:        gvk,
		groupResource: schema.GroupResource{
			Group:    gvk.Group,
			Resource: resource,
		},
	}
}

func FromRoleHandlerToHandler(sync RoleHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1.Role
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1.Role))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *roleController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1.Role))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateRoleDeepCopyOnChange(client RoleClient, obj *v1.Role, handler func(obj *v1.Role) (*v1.Role, error)) (*v1.Role, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *roleController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controller.RegisterHandler(ctx, name, controller.SharedControllerHandlerFunc(handler))
}

func (c *roleController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), handler))
}

func (c *roleController) OnChange(ctx context.Context, name string, sync RoleHandler) {
	c.AddGenericHandler(ctx, name, FromRoleHandlerToHandler(sync))
}

func (c *roleController) OnRemove(ctx context.Context, name string, sync RoleHandler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), FromRoleHandlerToHandler(sync)))
}

func (c *roleController) Enqueue(namespace, name string) {
	c.controller.Enqueue(namespace, name)
}

func (c *roleController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controller.EnqueueAfter(namespace, name, duration)
}

func (c *roleController) Informer() cache.SharedIndexInformer {
	return c.controller.Informer()
}

func (c *roleController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *roleController) Cache() RoleCache {
	return &roleCache{
		indexer:  c.Informer().GetIndexer(),
		resource: c.groupResource,
	}
}

func (c *roleController) Create(obj *v1.Role) (*v1.Role, error) {
	result := &v1.Role{}
	return result, c.client.Create(context.TODO(), obj.Namespace, obj, result, metav1.CreateOptions{})
}

func (c *roleController) Update(obj *v1.Role) (*v1.Role, error) {
	result := &v1.Role{}
	return result, c.client.Update(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *roleController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.client.Delete(context.TODO(), namespace, name, *options)
}

func (c *roleController) Get(namespace, name string, options metav1.GetOptions) (*v1.Role, error) {
	result := &v1.Role{}
	return result, c.client.Get(context.TODO(), namespace, name, result, options)
}

func (c *roleController) List(namespace string, opts metav1.ListOptions) (*v1.RoleList, error) {
	result := &v1.RoleList{}
	return result, c.client.List(context.TODO(), namespace, result, opts)
}

func (c *roleController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.client.Watch(context.TODO(), namespace, opts)
}

func (c *roleController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (*v1.Role, error) {
	result := &v1.Role{}
	return result, c.client.Patch(context.TODO(), namespace, name, pt, data, result, metav1.PatchOptions{}, subresources...)
}

type roleCache struct {
	indexer  cache.Indexer
	resource schema.GroupResource
}

func (c *roleCache) Get(namespace, name string) (*v1.Role, error) {
	obj, exists, err := c.indexer.GetByKey(namespace + "/" + name)
	if err != nil {
		return nil, err
	}
	if !exists {
		return nil, errors.NewNotFound(c.resource, name)
	}
	return obj.(*v1.Role), nil
}

func (c *roleCache) List(namespace string, selector labels.Selector) (ret []*v1.Role, err error) {

	err = cache.ListAllByNamespace(c.indexer, namespace, selector, func(m interface{}) {
		ret = append(ret, m.(*v1.Role))
	})

	return ret, err
}

func (c *roleCache) AddIndexer(indexName string, indexer RoleIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1.Role))
		},
	}))
}

func (c *roleCache) GetByIndex(indexName, key string) (result []*v1.Role, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v1.Role, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v1.Role))
	}
	return result, nil
}
